@@ -547,7 +547,8 @@ class CmdStanArgs:
         output_dir: str = None,
         sig_figs: str = None,
         save_diagnostics: bool = False,
-        refresh: str = None,
+        save_profile: bool = False,
+        refresh: int = None,
         logger: logging.Logger = None,
     ) -> None:
         """Initialize object."""
@@ -560,6 +561,7 @@ class CmdStanArgs:
         self.output_dir = output_dir
         self.sig_figs = sig_figs
         self.save_diagnostics = save_diagnostics
+        self.save_profile = save_profile
         self.refresh = refresh
         self.method_args = method_args
         if isinstance(method_args, SamplerArgs):
@@ -626,6 +628,12 @@ class CmdStanArgs:
                     'invalid path for output files,'
                     ' cannot write to dir: {}'.format(self.output_dir)
                 ) from exc
+        if self.refresh is not None:
+            if not isinstance(self.refresh, int) or self.refresh < 1:
+                raise ValueError(
+                    'Argument refresh must be a positive integer value, '
+                    'found {}.'.format(self.refresh)
+                )
 
         if self.sig_figs is not None:
             if (
@@ -685,10 +693,7 @@ class CmdStanArgs:
         if isinstance(self.data, str):
             if not os.path.exists(self.data):
                 raise ValueError('no such file {}'.format(self.data))
-        elif self.data is None:
-            if isinstance(self.method_args, OptimizeArgs):
-                raise ValueError('data must be set when optimizing')
-        elif not isinstance(self.data, (str, dict)):
+        elif self.data is not None and not isinstance(self.data, (str, dict)):
             raise ValueError('data must be string or dict')
 
         if self.inits is not None:
@@ -726,7 +731,12 @@ class CmdStanArgs:
                         )
 
     def compose_command(
-        self, idx: int, csv_file: str, diagnostic_file: str = None
+        self,
+        idx: int,
+        csv_file: str,
+        *,
+        diagnostic_file: str = None,
+        profile_file: str = None
     ) -> str:
         """
         Compose CmdStan command for non-default arguments.
@@ -763,6 +773,8 @@ class CmdStanArgs:
         cmd.append('file={}'.format(csv_file))
         if diagnostic_file is not None:
             cmd.append('diagnostic_file={}'.format(diagnostic_file))
+        if profile_file is not None:
+            cmd.append('profile_file={}'.format(profile_file))
         if self.refresh is not None:
             cmd.append('refresh={}'.format(self.refresh))
         if self.sig_figs is not None:
