@@ -342,9 +342,11 @@ class CmdStanModel:
         inits: Union[Dict, float, str] = None,
         output_dir: str = None,
         sig_figs: int = None,
+        save_profile: bool = False,
         algorithm: str = None,
         init_alpha: float = None,
         iter: int = None,
+        refresh: int = None,
     ) -> CmdStanMLE:
         """
         Run the specified CmdStan optimize algorithm to produce a
@@ -400,12 +402,20 @@ class CmdStanModel:
             precision for the system file I/O is used; the usual value is 6.
             Introduced in CmdStan-2.25.
 
+        :param save_profile: Whether or not to profile auto-diff operations in
+            labelled blocks of code.  If True, csv outputs are written to a file
+            '<model_name>-<YYYYMMDDHHMM>-profile-<chain_id>'.
+            Introduced in CmdStan-2.26.
+
         :param algorithm: Algorithm to use. One of: 'BFGS', 'LBFGS', 'Newton'
 
         :param init_alpha: Line search step size for first iteration
 
         :param iter: Total number of iterations
 
+        :param refresh: Specify the number of iterations cmdstan will take
+        between progress messages. Default value is 100.
+
         :return: CmdStanMLE object
         """
         optimize_args = OptimizeArgs(
@@ -422,8 +432,9 @@ class CmdStanModel:
                 inits=_inits,
                 output_dir=output_dir,
                 sig_figs=sig_figs,
-                save_diagnostics=False,
+                save_profile=save_profile,
                 method_args=optimize_args,
+                refresh=refresh,
             )
 
             dummy_chain_id = 0
@@ -462,8 +473,10 @@ class CmdStanModel:
         output_dir: str = None,
         sig_figs: int = None,
         save_diagnostics: bool = False,
+        save_profile: bool = False,
         show_progress: Union[bool, str] = False,
         validate_csv: bool = True,
+        refresh: int = None,
     ) -> CmdStanMCMC:
         """
         Run or more chains of the NUTS sampler to produce a set of draws
@@ -605,11 +618,17 @@ class CmdStanModel:
             precision for the system file I/O is used; the usual value is 6.
             Introduced in CmdStan-2.25.
 
-        :param save_diagnostics: Whether or not to save diagnostics. If True,
-            csv output files are written to an output file with filename
+        :param save_diagnostics: Whether or not to output the position and
+            momentum information for each parameter.  If True,
+            csv outputs are written to an output file using filename
             template '<model_name>-<YYYYMMDDHHMM>-diagnostic-<chain_id>',
             e.g. 'bernoulli-201912081451-diagnostic-1.csv'.
 
+        :param save_profile: Whether or not to profile auto-diff operations in
+            labelled blocks of code.  If True, csv outputs are written to a file
+            '<model_name>-<YYYYMMDDHHMM>-profile-<chain_id>'.
+            Introduced in CmdStan-2.26.
+
         :param show_progress: Use tqdm progress bar to show sampling progress.
             If show_progress=='notebook' use tqdm_notebook
             (needs nodejs for jupyter).
@@ -619,6 +638,9 @@ class CmdStanModel:
             Default is ``True`` - sample csv files are scanned for completeness
             and consistency.
 
+        :param refresh: Specify the number of iterations cmdstan will take
+        between progress messages. Default value is 100.
+
         :return: CmdStanMCMC object
         """
         if chains is None:
@@ -683,7 +705,6 @@ class CmdStanModel:
         )
         os.environ['STAN_NUM_THREADS'] = str(threads_per_chain)
 
-        refresh = None
         if show_progress:
             try:
                 import tqdm
@@ -727,6 +748,7 @@ class CmdStanModel:
                 output_dir=output_dir,
                 sig_figs=sig_figs,
                 save_diagnostics=save_diagnostics,
+                save_profile=save_profile,
                 method_args=sampler_args,
                 refresh=refresh,
                 logger=self._logger,
@@ -798,6 +820,7 @@ class CmdStanModel:
         seed: int = None,
         gq_output_dir: str = None,
         sig_figs: int = None,
+        refresh: int = None,
     ) -> CmdStanGQ:
         """
         Run CmdStan's generate_quantities method which runs the generated
@@ -846,6 +869,9 @@ class CmdStanModel:
             precision for the system file I/O is used; the usual value is 6.
             Introduced in CmdStan-2.25.
 
+        :param refresh: Specify the number of iterations cmdstan will take
+            between progress messages. Default value is 100.
+
         :return: CmdStanGQ object
         """
         sample_csv_files = []
@@ -922,6 +948,7 @@ class CmdStanModel:
                 output_dir=gq_output_dir,
                 sig_figs=sig_figs,
                 method_args=generate_quantities_args,
+                refresh=refresh,
             )
             runset = RunSet(args=args, chains=chains, chain_ids=chain_ids)
 
@@ -947,6 +974,7 @@ class CmdStanModel:
         output_dir: str = None,
         sig_figs: int = None,
         save_diagnostics: bool = False,
+        save_profile: bool = False,
         algorithm: str = None,
         iter: int = None,
         grad_samples: int = None,
@@ -958,6 +986,7 @@ class CmdStanModel:
         eval_elbo: int = None,
         output_samples: int = None,
         require_converged: bool = True,
+        refresh: int = None,
     ) -> CmdStanVB:
         """
         Run CmdStan's variational inference algorithm to approximate
@@ -1006,10 +1035,15 @@ class CmdStanModel:
             Introduced in CmdStan-2.25.
 
         :param save_diagnostics: Whether or not to save diagnostics. If True,
-            csv output files are written to an output file with filename
+            csv outputs are written to an output file using filename
             template '<model_name>-<YYYYMMDDHHMM>-diagnostic-<chain_id>',
             e.g. 'bernoulli-201912081451-diagnostic-1.csv'.
 
+        :param save_profile: Whether or not to profile auto-diff operations in
+            labelled blocks of code.  If True, csv outputs are written to a file
+            '<model_name>-<YYYYMMDDHHMM>-profile-<chain_id>'.
+            Introduced in CmdStan-2.26.
+
         :param algorithm: Algorithm to use. One of: 'meanfield', 'fullrank'.
 
         :param iter: Maximum number of ADVI iterations.
@@ -1034,6 +1068,9 @@ class CmdStanModel:
         :param require_converged: Whether or not to raise an error if stan
             reports that "The algorithm may not have converged".
 
+        :param refresh: Specify the number of iterations cmdstan will take
+            between progress messages. Default value is 100.
+
         :return: CmdStanVB object
         """
         variational_args = VariationalArgs(
@@ -1060,7 +1097,9 @@ class CmdStanModel:
                 output_dir=output_dir,
                 sig_figs=sig_figs,
                 save_diagnostics=save_diagnostics,
+                save_profile=save_profile,
                 method_args=variational_args,
+                refresh=refresh,
             )
 
             dummy_chain_id = 0
@@ -1101,7 +1140,11 @@ class CmdStanModel:
         )
         self._logger.debug('sampling: %s', cmd)
         proc = subprocess.Popen(
-            cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=os.environ
+            cmd,
+            stdin=subprocess.DEVNULL,
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE,
+            env=os.environ,
         )
         if pbar:
             stdout_pbar = self._read_progress(proc, pbar, idx)
