@@ -79,6 +79,7 @@ class RunSet:
             output_dir = _TMPDIR
         self._csv_files = [None for _ in range(chains)]
         self._diagnostic_files = [None for _ in range(chains)]
+        self._profile_files = [None for _ in range(chains)]
         self._stdout_files = [None for _ in range(chains)]
         self._stderr_files = [None for _ in range(chains)]
         self._cmds = []
@@ -103,6 +104,7 @@ class RunSet:
                 [os.path.splitext(csv_file)[0], '-stderr.txt']
             )
             self._stderr_files[i] = stderr_file
+            # optional output files:  diagnostics, profiling
             if args.save_diagnostics:
                 if args.output_dir is None:
                     diag_file = create_named_text_file(
@@ -120,9 +122,46 @@ class RunSet:
                         ),
                     )
                 self._diagnostic_files[i] = diag_file
+            if args.save_profile:
+                if args.output_dir is None:
+                    profile_file = create_named_text_file(
+                        dir=_TMPDIR,
+                        prefix='{}-profile-{}-'.format(
+                            file_basename, str(chain_ids[i])
+                        ),
+                        suffix='.csv',
+                    )
+                else:
+                    profile_file = os.path.join(
+                        output_dir,
+                        '{}-profile-{}.{}'.format(
+                            file_basename, str(chain_ids[i]), 'csv'
+                        ),
+                    )
+                self._profile_files[i] = profile_file
+            if args.save_diagnostics and args.save_profile:
                 self._cmds.append(
                     args.compose_command(
-                        i, self._csv_files[i], self._diagnostic_files[i]
+                        i,
+                        self._csv_files[i],
+                        diagnostic_file=self._diagnostic_files[i],
+                        profile_file=self._profile_files[i],
+                    )
+                )
+            elif args.save_diagnostics:
+                self._cmds.append(
+                    args.compose_command(
+                        i,
+                        self._csv_files[i],
+                        diagnostic_file=self._diagnostic_files[i],
+                    )
+                )
+            elif args.save_profile:
+                self._cmds.append(
+                    args.compose_command(
+                        i,
+                        self._csv_files[i],
+                        profile_file=self._profile_files[i],
                     )
                 )
             else:
@@ -142,6 +181,10 @@ class RunSet:
             repr = '{}\n diagnostics_files:\n\t{}'.format(
                 repr, '\n\t'.join(self._diagnostic_files)
             )
+        if self._args.save_profile and os.path.exists(self._profile_files[0]):
+            repr = '{}\n profile_files:\n\t{}'.format(
+                repr, '\n\t'.join(self._profile_files)
+            )
         if os.path.exists(self._stdout_files[0]):
             repr = '{}\n console_msgs:\n\t{}'.format(
                 repr, '\n\t'.join(self._stdout_files)
@@ -201,9 +244,14 @@ class RunSet:
 
     @property
     def diagnostic_files(self) -> List[str]:
-        """List of paths to CmdStan diagnostic output files."""
+        """List of paths to CmdStan hamiltonian diagnostic files."""
         return self._diagnostic_files
 
+    @property
+    def profile_files(self) -> List[str]:
+        """List of paths to CmdStan profiler files."""
+        return self._profile_files
+
     def _retcode(self, idx: int) -> int:
         """Get retcode for chain[idx]."""
         return self._retcodes[idx]
@@ -378,7 +426,7 @@ class CmdStanMCMC:
         self._step_size = None
         # inference
         self._draws = None
-        self._draws_pd = None  # slated for removal
+        self._draws_pd = None
         self._validate_csv = validate_csv
         if validate_csv:
             self.validate_csv_files()
@@ -394,6 +442,7 @@ class CmdStanMCMC:
             '\n\t'.join(self.runset.csv_files),
             '\n\t'.join(self.runset.stdout_files),
         )
+        # TODO - hamiltonian, profiling files
         return repr
 
     @property
@@ -606,7 +655,7 @@ class CmdStanMCMC:
         if concat_chains:
             num_rows *= self.chains
             return self._draws[start_idx:, :, :].reshape(
-                (num_rows, len(self.column_names)), order='A'
+                (num_rows, len(self.column_names)), order='F'
             )
         return self._draws[start_idx:, :, :]
 
@@ -665,6 +714,7 @@ class CmdStanMCMC:
                             'id',
                             'diagnostic_file',
                             'metric_file',
+                            'profile_file',
                             'stepsize',
                             'init',
                             'seed',
@@ -872,10 +922,7 @@ class CmdStanMCMC:
         :param inc_warmup: When ``True`` and the warmup draws are present in
             the output, i.e., the sampler was run with ``save_warmup=True``,
             then the warmup draws are included.  Default value is ``False``.
-
-        *Note:* Slated for removal
         """
-        self._logger.warning('method "draws_pd" is slated for removal.')
         if inc_warmup and not self._save_warmup:
             self._logger.warning(
                 'draws from warmup iterations not available,'
@@ -902,7 +949,7 @@ class CmdStanMCMC:
         if self._draws_pd is None or self._draws_pd.shape[0] != num_rows:
             # pylint: disable=redundant-keyword-arg
             data = self.draws(inc_warmup=inc_warmup).reshape(
-                (num_rows, len(self.column_names)), order='A'
+                (num_rows, len(self.column_names)), order='F'
             )
             self._draws_pd = pd.DataFrame(data=data, columns=self.column_names)
         if params is None:
@@ -957,7 +1004,7 @@ class CmdStanMCMC:
         if len(col_idxs) > 0:
             dims.extend(self._metadata.stan_vars_dims[name])
         # pylint: disable=redundant-keyword-arg
-        return self._draws[draw1:, :, col_idxs].reshape(dims, order='A')
+        return self._draws[draw1:, :, col_idxs].reshape(dims, order='F')
 
     def stan_variables(self) -> Dict:
         """
@@ -1028,6 +1075,7 @@ class CmdStanMLE:
             '\n\t'.join(self.runset.csv_files),
             '\n\t'.join(self.runset.stdout_files),
         )
+        # TODO - profiling files
         return repr
 
     def _set_mle_attrs(self, sample_csv_0: str) -> None:
@@ -1222,6 +1270,7 @@ class CmdStanVB:
             '\n\t'.join(self.runset.csv_files),
             '\n\t'.join(self.runset.stdout_files),
         )
+        # TODO - diagnostic, profiling files
         return repr
 
     def _set_variational_attrs(self, sample_csv_0: str) -> None:
