@@ -1,30 +1,32 @@
 """
 Utility functions
 """
+import logging
+import math
 import os
+import platform
+import re
+import shutil
+import subprocess
+import sys
+import tempfile
 from collections import OrderedDict
 from collections.abc import Sequence
 from numbers import Integral, Real
-
-try:
-    import ujson as json
-except ImportError:
-    import json
-import math
-import platform
-import re
-import subprocess
-import shutil
-import tempfile
-import logging
-import sys
-from typing import Dict, TextIO, List, Union, Tuple
+from typing import Dict, List, TextIO, Tuple, Union
 
 import numpy as np
 import pandas as pd
+import ujson as json
 
-
-from cmdstanpy import _TMPDIR, _CMDSTAN_WARMUP, _CMDSTAN_SAMPLING, _CMDSTAN_THIN
+from cmdstanpy import (
+    _CMDSTAN_SAMPLING,
+    _CMDSTAN_THIN,
+    _CMDSTAN_WARMUP,
+    _DOT_CMDSTAN,
+    _DOT_CMDSTANPY,
+    _TMPDIR,
+)
 
 EXTENSION = '.exe' if platform.system() == 'Windows' else ''
 
@@ -37,74 +39,68 @@ def get_logger():
     return logger
 
 
-def get_latest_cmdstan(dot_dir: str) -> str:
+def validate_dir(install_dir: str):
+    """Check that specified install directory exists, can write."""
+    if not os.path.exists(install_dir):
+        try:
+            os.makedirs(install_dir)
+        except (IOError, OSError, PermissionError) as e:
+            raise ValueError(
+                'Cannot create directory: {}'.format(install_dir)
+            ) from e
+    else:
+        if not os.path.isdir(install_dir):
+            raise ValueError(
+                'File exists, should be a directory: {}'.format(install_dir)
+            )
+        try:
+            with open('tmp_test_w', 'w'):
+                pass
+            os.remove('tmp_test_w')  # cleanup
+        except OSError as e:
+            raise ValueError(
+                'Cannot write files to directory {}'.format(install_dir)
+            ) from e
+
+
+def get_latest_cmdstan(cmdstan_dir: str) -> str:
     """
     Given a valid directory path, find all installed CmdStan versions
     and return highest (i.e., latest) version number.
     Assumes directory populated via script `install_cmdstan`.
     """
     versions = [
-        name.split('-')[1]
-        for name in os.listdir(dot_dir)
-        if os.path.isdir(os.path.join(dot_dir, name))
+        ''.join(name.split('-')[1:])  # name may contain '-rc'
+        for name in os.listdir(cmdstan_dir)
+        if os.path.isdir(os.path.join(cmdstan_dir, name))
         and name.startswith('cmdstan-')
         and name[8].isdigit()
     ]
+    # munge rc for sort, e.g. 2.25.0-rc1 -> 2.25.0.-99
+    for i in range(len(versions)):  # # pylint: disable=C0200
+        tmp = versions[i].split('rc')
+        if len(tmp) == 1:
+            versions[i] = '.'.join([tmp[0], '0'])
+        else:
+            rc_sortable = str(int(tmp[1]) - 100)
+            versions[i] = '.'.join([tmp[0], rc_sortable])
+
     versions.sort(key=lambda s: list(map(int, s.split('.'))))
     if len(versions) == 0:
         return None
     latest = 'cmdstan-{}'.format(versions[len(versions) - 1])
+
+    # unmunge
+    tmp = latest.split('.')
+    prefix = '.'.join(tmp[0:3])
+    if int(tmp[3]) == 0:
+        latest = prefix
+    else:
+        tmp[3] = 'rc' + str(int(tmp[3]) + 100)
+        latest = '-'.join([prefix, tmp[3]])
     return latest
 
 
-class MaybeDictToFilePath:
-    """Context manager for json files."""
-
-    def __init__(self, *objs: Union[str, dict], logger: logging.Logger = None):
-        self._unlink = [False] * len(objs)
-        self._paths = [''] * len(objs)
-        self._logger = logger or get_logger()
-        i = 0
-        for obj in objs:
-            if isinstance(obj, dict):
-                data_file = create_named_text_file(
-                    dir=_TMPDIR, prefix='', suffix='.json'
-                )
-                self._logger.debug('input tempfile: %s', data_file)
-                if any(
-                    not item
-                    for item in obj
-                    if isinstance(item, (Sequence, np.ndarray))
-                ):
-                    rdump(data_file, obj)
-                else:
-                    jsondump(data_file, obj)
-                self._paths[i] = data_file
-                self._unlink[i] = True
-            elif isinstance(obj, str):
-                if not os.path.exists(obj):
-                    raise ValueError("File doesn't exist {}".format(obj))
-                self._paths[i] = obj
-            elif obj is None:
-                self._paths[i] = None
-            elif i == 1 and isinstance(obj, (Integral, Real)):
-                self._paths[i] = obj
-            else:
-                raise ValueError('data must be string or dict')
-            i += 1
-
-    def __enter__(self):
-        return self._paths
-
-    def __exit__(self, exc_type, exc_val, exc_tb):
-        for can_unlink, path in zip(self._unlink, self._paths):
-            if can_unlink and path:
-                try:
-                    os.remove(path)
-                except PermissionError:
-                    pass
-
-
 def validate_cmdstan_path(path: str) -> None:
     """
     Validate that CmdStan directory exists and binaries have been built.
@@ -119,46 +115,6 @@ def validate_cmdstan_path(path: str) -> None:
         )
 
 
-class TemporaryCopiedFile:
-    """Context manager for tmpfiles, handles spaces in filepath."""
-
-    def __init__(self, file_path: str):
-        self._path = None
-        self._tmpdir = None
-        if ' ' in os.path.abspath(file_path) and platform.system() == 'Windows':
-            base_path, file_name = os.path.split(os.path.abspath(file_path))
-            os.makedirs(base_path, exist_ok=True)
-            try:
-                short_base_path = windows_short_path(base_path)
-                if os.path.exists(short_base_path):
-                    file_path = os.path.join(short_base_path, file_name)
-            except RuntimeError:
-                pass
-
-        if ' ' in os.path.abspath(file_path):
-            tmpdir = tempfile.mkdtemp()
-            if ' ' in tmpdir:
-                raise RuntimeError(
-                    'Unable to generate temporary path without spaces! \n'
-                    + 'Please move your stan file to location without spaces.'
-                )
-
-            _, path = tempfile.mkstemp(suffix='.stan', dir=tmpdir)
-
-            shutil.copy(file_path, path)
-            self._path = path
-            self._tmpdir = tmpdir
-        else:
-            self._path = file_path
-
-    def __enter__(self):
-        return self._path, self._tmpdir is not None
-
-    def __exit__(self, exc_type, exc_val, exc_tb):
-        if self._tmpdir:
-            shutil.rmtree(self._tmpdir, ignore_errors=True)
-
-
 def set_cmdstan_path(path: str) -> None:
     """
     Validate, then set CmdStan directory path.
@@ -182,12 +138,14 @@ def cmdstan_path() -> str:
     if 'CMDSTAN' in os.environ:
         cmdstan = os.environ['CMDSTAN']
     else:
-        cmdstan_dir = os.path.expanduser(os.path.join('~', '.cmdstanpy'))
+        cmdstan_dir = os.path.expanduser(os.path.join('~', _DOT_CMDSTAN))
         if not os.path.exists(cmdstan_dir):
-            raise ValueError(
-                'no CmdStan installation found, '
-                'run command line script "install_cmdstan"'
-            )
+            cmdstan_dir = os.path.expanduser(os.path.join('~', _DOT_CMDSTANPY))
+            if not os.path.exists(cmdstan_dir):
+                raise ValueError(
+                    'no CmdStan installation found, '
+                    'run command line script "install_cmdstan"'
+                )
         latest_cmdstan = get_latest_cmdstan(cmdstan_dir)
         if latest_cmdstan is None:
             raise ValueError(
@@ -200,6 +158,51 @@ def cmdstan_path() -> str:
     return cmdstan
 
 
+def cmdstan_version_at(maj: int, min: int) -> bool:
+    """
+    Check that CmdStan version is at or above Maj.min version.
+    Parses version string out of CmdStan makefile in CmdStan path dir.
+
+    :param maj: Major version number
+    :param min: Minor version number
+
+    :return: True if version at or above, else False
+    """
+    # pylint:disable=bare-except
+    try:
+        path = cmdstan_path()
+        makefile = os.path.join(path, 'makefile')
+        if not os.path.exists(makefile):
+            raise ValueError(
+                'CmdStan installation {}: missing makefile'.format(path)
+            )
+        version = None
+        with open(makefile, 'r') as fd:
+            contents = fd.read()
+            start_idx = contents.find('CMDSTAN_VERSION := ') + len(
+                'CMDSTAN_VERSION := '
+            )
+            end_idx = contents.find('\n', start_idx)
+            version = contents[start_idx:end_idx]
+        if version is None:
+            raise ValueError(
+                'Cannot parse version from makefile: {}'.format(makefile)
+            )
+        splits = version.split('.')
+        if len(splits) < 2:
+            raise ValueError(
+                'Cannot parse version from makefile: {}'.format(makefile)
+            )
+        cur_maj = int(splits[0])
+        cur_min = int(splits[1])
+
+        if cur_maj > maj or (cur_maj == maj and cur_min >= min):
+            return True
+    except:  # noqa
+        pass
+    return False
+
+
 def cxx_toolchain_path(version: str = None) -> Tuple[str]:
     """
     Validate, then activate C++ toolchain directory path.
@@ -211,7 +214,6 @@ def cxx_toolchain_path(version: str = None) -> Tuple[str]:
     if version is not None and not isinstance(version, str):
         raise TypeError('Format version number as a string')
     logger = get_logger()
-    toolchain_root = ''
     if 'CMDSTAN_TOOLCHAIN' in os.environ:
         toolchain_root = os.environ['CMDSTAN_TOOLCHAIN']
         if os.path.exists(os.path.join(toolchain_root, 'mingw64')):
@@ -229,14 +231,14 @@ def cxx_toolchain_path(version: str = None) -> Tuple[str]:
                         'Found invalid installion for RTools40 on %s',
                         toolchain_root,
                     )
-                    toolchain_root = ''
+                    toolchain_root = None
             else:
                 compiler_path = ''
                 logger.warning(
                     'Found invalid installion for RTools40 on %s',
                     toolchain_root,
                 )
-                toolchain_root = ''
+                toolchain_root = None
 
         elif os.path.exists(os.path.join(toolchain_root, 'mingw_64')):
             compiler_path = os.path.join(
@@ -253,93 +255,94 @@ def cxx_toolchain_path(version: str = None) -> Tuple[str]:
                         'Found invalid installion for RTools35 on %s',
                         toolchain_root,
                     )
-                    toolchain_root = ''
+                    toolchain_root = None
             else:
                 compiler_path = ''
                 logger.warning(
                     'Found invalid installion for RTools35 on %s',
                     toolchain_root,
                 )
-                toolchain_root = ''
+                toolchain_root = None
     else:
-        rtools_dir = os.path.expanduser(
-            os.path.join('~', '.cmdstanpy', 'RTools40')
-        )
-        if not os.path.exists(rtools_dir):
-            rtools_dir = os.path.expanduser(
-                os.path.join('~', '.cmdstanpy', 'RTools35')
-            )
-            if not os.path.exists(rtools_dir):
-                rtools_dir = os.path.expanduser(
-                    os.path.join('~', '.cmdstanpy', 'RTools')
-                )
-                if not os.path.exists(rtools_dir):
-                    raise ValueError(
-                        'no RTools installation found, '
-                        'run command line script "install_cxx_toolchain"'
-                    )
-            else:
-                rtools_dir = os.path.expanduser(os.path.join('~', '.cmdstanpy'))
-        else:
-            rtools_dir = os.path.expanduser(os.path.join('~', '.cmdstanpy'))
-        compiler_path = ''
-        tool_path = ''
-        if version not in ('35', '3.5', '3') and os.path.exists(
-            os.path.join(rtools_dir, 'RTools40')
-        ):
-            toolchain_root = os.path.join(rtools_dir, 'RTools40')
-            compiler_path = os.path.join(
-                toolchain_root,
-                'mingw64' if (sys.maxsize > 2 ** 32) else 'mingw32',
-                'bin',
-            )
-            if os.path.exists(compiler_path):
-                tool_path = os.path.join(toolchain_root, 'usr', 'bin')
-                if not os.path.exists(tool_path):
-                    tool_path = ''
-                    compiler_path = ''
-                    logger.warning(
-                        'Found invalid installation for RTools40 on %s',
+        rtools40_home = os.environ.get('RTOOLS40_HOME')
+        cmdstan_dir = os.path.expanduser(os.path.join('~', _DOT_CMDSTAN))
+        cmdstan_dir_old = os.path.expanduser(os.path.join('~', _DOT_CMDSTANPY))
+        for toolchain_root in (
+            [rtools40_home] if rtools40_home is not None else []
+        ) + [
+            os.path.join(cmdstan_dir, 'RTools40'),
+            os.path.join(cmdstan_dir_old, 'RTools40'),
+            os.path.join(os.path.abspath("/"), "RTools40"),
+            os.path.join(cmdstan_dir, 'RTools35'),
+            os.path.join(cmdstan_dir_old, 'RTools35'),
+            os.path.join(os.path.abspath("/"), "RTools35"),
+            os.path.join(cmdstan_dir, 'RTools'),
+            os.path.join(cmdstan_dir_old, 'RTools'),
+            os.path.join(os.path.abspath("/"), "RTools"),
+            os.path.join(os.path.abspath("/"), "RBuildTools"),
+        ]:
+            compiler_path = ''
+            tool_path = ''
+
+            if os.path.exists(toolchain_root):
+                if version not in ('35', '3.5', '3'):
+                    compiler_path = os.path.join(
                         toolchain_root,
+                        'mingw64' if (sys.maxsize > 2 ** 32) else 'mingw32',
+                        'bin',
                     )
-                    toolchain_root = ''
-            else:
-                compiler_path = ''
-                logger.warning(
-                    'Found invalid installation for RTools40 on %s',
-                    toolchain_root,
-                )
-                toolchain_root = ''
-        if (
-            not toolchain_root or version in ('4', '40', '4.0')
-        ) and os.path.exists(os.path.join(rtools_dir, 'RTools35')):
-            toolchain_root = os.path.join(rtools_dir, 'RTools35')
-            compiler_path = os.path.join(
-                toolchain_root,
-                'mingw_64' if (sys.maxsize > 2 ** 32) else 'mingw_32',
-                'bin',
-            )
-            if os.path.exists(compiler_path):
-                tool_path = os.path.join(toolchain_root, 'bin')
-                if not os.path.exists(tool_path):
-                    tool_path = ''
-                    compiler_path = ''
-                    logger.warning(
-                        'Found invalid installation for RTools35 on %s',
+                    if os.path.exists(compiler_path):
+                        tool_path = os.path.join(toolchain_root, 'usr', 'bin')
+                        if not os.path.exists(tool_path):
+                            tool_path = ''
+                            compiler_path = ''
+                            logger.warning(
+                                'Found invalid installation for RTools40 on %s',
+                                toolchain_root,
+                            )
+                            toolchain_root = None
+                        else:
+                            break
+                    else:
+                        compiler_path = ''
+                        logger.warning(
+                            'Found invalid installation for RTools40 on %s',
+                            toolchain_root,
+                        )
+                        toolchain_root = None
+                else:
+                    compiler_path = os.path.join(
                         toolchain_root,
+                        'mingw_64' if (sys.maxsize > 2 ** 32) else 'mingw_32',
+                        'bin',
                     )
-                    toolchain_root = ''
+                    if os.path.exists(compiler_path):
+                        tool_path = os.path.join(toolchain_root, 'bin')
+                        if not os.path.exists(tool_path):
+                            tool_path = ''
+                            compiler_path = ''
+                            logger.warning(
+                                'Found invalid installation for RTools35 on %s',
+                                toolchain_root,
+                            )
+                            toolchain_root = None
+                        else:
+                            break
+                    else:
+                        compiler_path = ''
+                        logger.warning(
+                            'Found invalid installation for RTools35 on %s',
+                            toolchain_root,
+                        )
+                        toolchain_root = None
             else:
-                compiler_path = ''
-                logger.warning(
-                    'Found invalid installation for RTools35 on %s',
-                    toolchain_root,
-                )
-                toolchain_root = ''
+                toolchain_root = None
+
     if not toolchain_root:
         raise ValueError(
-            'no C++ toolchain installation found, '
-            'run command line script "install_cxx_toolchain"'
+            'no RTools toolchain installation found, '
+            'run command line script '
+            '"python -m cmdstanpy.install_cxx_toolchain"'
         )
     logger.info('Add C++ toolchain to $PATH: %s', toolchain_root)
     os.environ['PATH'] = ';'.join(
@@ -367,7 +370,7 @@ def jsondump(path: str, data: Dict) -> None:
     """Dump a dict of data to a JSON file."""
     data = data.copy()
     for key, val in data.items():
-        if isinstance(val, np.ndarray):
+        if type(val).__module__ == 'numpy':
             val = val.tolist()
             data[key] = val
     with open(path, 'w') as fd:
@@ -444,8 +447,8 @@ def parse_rdump_value(rhs: str) -> Union[int, float, np.array]:
             val = float(rhs)
         else:
             val = int(rhs)
-    except TypeError as exc:
-        raise ValueError('bad value in Rdump file: {}'.format(rhs)) from exc
+    except TypeError as e:
+        raise ValueError('bad value in Rdump file: {}'.format(rhs)) from e
     return val
 
 
@@ -490,7 +493,6 @@ def check_sampler_csv(
         )
     if save_warmup:
         if not ('save_warmup' in meta and meta['save_warmup'] == 1):
-            print(meta)
             raise ValueError(
                 'bad csv file {}, '
                 'config error, expected save_warmup = 1'.format(path)
@@ -586,24 +588,23 @@ def scan_config(fd: TextIO, config_dict: Dict, lineno: int) -> int:
     line = fd.readline().strip()
     while len(line) > 0 and line.startswith('#'):
         lineno += 1
-        if not line.endswith('(Default)'):
-            line = line.lstrip(' #\t')
-            key_val = line.split('=')
-            if len(key_val) == 2:
-                if key_val[0].strip() == 'file' and not key_val[1].endswith(
-                    'csv'
-                ):
-                    config_dict['data_file'] = key_val[1].strip()
-                elif key_val[0].strip() != 'file':
-                    raw_val = key_val[1].strip()
+        if line.endswith('(Default)'):
+            line = line.replace('(Default)', '')
+        line = line.lstrip(' #\t')
+        key_val = line.split('=')
+        if len(key_val) == 2:
+            if key_val[0].strip() == 'file' and not key_val[1].endswith('csv'):
+                config_dict['data_file'] = key_val[1].strip()
+            elif key_val[0].strip() != 'file':
+                raw_val = key_val[1].strip()
+                try:
+                    val = int(raw_val)
+                except ValueError:
                     try:
-                        val = int(raw_val)
+                        val = float(raw_val)
                     except ValueError:
-                        try:
-                            val = float(raw_val)
-                        except ValueError:
-                            val = raw_val
-                    config_dict[key_val[0].strip()] = val
+                        val = raw_val
+                config_dict[key_val[0].strip()] = val
         cur_pos = fd.tell()
         line = fd.readline().strip()
     fd.seek(cur_pos)
@@ -636,40 +637,75 @@ def scan_column_names(fd: TextIO, config_dict: Dict, lineno: int) -> int:
     line = fd.readline().strip()
     lineno += 1
     names = line.split(',')
-    config_dict['column_names'] = tuple(names)
-    config_dict['num_params'] = len(names) - 1
+    config_dict['column_names_raw'] = tuple(names)
+    config_dict['column_names'] = tuple(munge_varnames(names))
     return lineno
 
 
-def parse_var_dims(names: Tuple[str, ...]) -> Dict:
+def munge_varnames(names: List) -> List:
     """
-    Use Stan CSV file column names to get variable names, dimensions.
-    Assumes that CSV file has been validated and column names are correct.
+    Change formatting for indices of container var elements
+    from use of dot separator to array-like notation, e.g.,
+    rewrite label ``y_forecast.2.4`` to ``y_forecast[2,4]``.
     """
     if names is None:
         raise ValueError('missing argument "names"')
-    vars_dict = {}
-    idx = 0
-    while idx < len(names):
-        if names[idx].endswith('__'):
-            pass
-        elif '.' not in names[idx]:
-            vars_dict[names[idx]] = 1
+    return [
+        re.sub(r',([\d,]+)$', r'[\1]', column.replace('.', ','))
+        for column in names
+    ]
+
+
+def parse_sampler_vars(names: Tuple[str, ...]) -> Dict:
+    """
+    Parses out names ending in `__` from list of CSV file column names.
+    Return a dict mapping sampler variable name to Stan CSV file column, using
+    zero-based column indexing.
+    """
+    if names is None:
+        raise ValueError('missing argument "names"')
+    # note: value as tuple allows structured sampler vars
+    # currently, all sampler vars a scalar, not checking for structure
+    return {v: tuple([k]) for (k, v) in enumerate(names) if v.endswith('__')}
+
+
+def parse_stan_vars(names: Tuple[str, ...]) -> (Dict, Dict):
+    """
+    Parses out Stan variable names (i.e., names not ending in `__`)
+    from list of CSV file column names.
+    Returns a pair of dicts which map variable names to dimensions and
+    variable names to columns, respectively, using zero-based column indexing.
+    Note: assumes: (a) munged varnames and (b) container vars are non-ragged
+    and dense; no checks size, indices.
+    """
+    if names is None:
+        raise ValueError('missing argument "names"')
+    dims_map = {}
+    cols_map = {}
+    idxs = []
+    for (idx, name) in enumerate(names):
+        idxs.append(idx)
+        var, *dims = name.split('[')
+        if var.endswith('__'):
+            idxs = []
+        elif len(dims) == 0:
+            dims_map[var] = ()
+            cols_map[var] = tuple(idxs)
+            idxs = []
         else:
-            vs = names[idx].split('.')
-            if idx < len(names) - 1 and names[idx + 1].split('.')[0] == vs[0]:
-                idx += 1
+            if idx < len(names) - 1 and names[idx + 1].split('[')[0] == var:
                 continue
-            dims = [int(vs[x]) for x in range(1, len(vs))]
-            vars_dict[vs[0]] = tuple(dims)
-        idx += 1
-    return vars_dict
+            dims = [int(x) for x in dims[0][:-1].split(',')]
+            dims_map[var] = tuple(dims)
+            cols_map[var] = tuple(idxs)
+            idxs = []
+    return (dims_map, cols_map)
 
 
 def scan_metric(fd: TextIO, config_dict: Dict, lineno: int) -> int:
     """
-    Scan stepsize, metric from  stan_csv file comment lines,
-    set config_dict entries 'metric' and 'num_params'
+    Scan step size, metric from  stan_csv file comment lines,
+    set config_dict entries 'metric' and 'num_unconstrained_params'
     """
     if 'metric' not in config_dict:
         config_dict['metric'] = 'diag_e'
@@ -682,18 +718,18 @@ def scan_metric(fd: TextIO, config_dict: Dict, lineno: int) -> int:
         )
     line = fd.readline().strip()
     lineno += 1
-    label, stepsize = line.split('=')
+    label, step_size = line.split('=')
     if not label.startswith('# Step size'):
         raise ValueError(
-            'line {}: expecting stepsize, '
+            'line {}: expecting step size, '
             'found:\n\t "{}"'.format(lineno, line)
         )
     try:
-        float(stepsize.strip())
-    except ValueError as exc:
+        float(step_size.strip())
+    except ValueError as e:
         raise ValueError(
-            'line {}: invalid stepsize: {}'.format(lineno, stepsize)
-        ) from exc
+            'line {}: invalid step size: {}'.format(lineno, step_size)
+        ) from e
     line = fd.readline().strip()
     lineno += 1
     if not (
@@ -711,15 +747,15 @@ def scan_metric(fd: TextIO, config_dict: Dict, lineno: int) -> int:
         )
     line = fd.readline().lstrip(' #\t')
     lineno += 1
-    num_params = len(line.split(','))
-    config_dict['num_params'] = num_params
+    num_unconstrained_params = len(line.split(','))
+    config_dict['num_unconstrained_params'] = num_unconstrained_params
     if metric == 'diag_e':
         return lineno
     else:
-        for _ in range(1, num_params):
+        for _ in range(1, num_unconstrained_params):
             line = fd.readline().lstrip(' #\t')
             lineno += 1
-            if len(line.split(',')) != num_params:
+            if len(line.split(',')) != num_unconstrained_params:
                 raise ValueError(
                     'line {}: invalid or missing mass matrix '
                     'specification'.format(lineno)
@@ -803,14 +839,16 @@ def do_command(cmd: str, cwd: str = None, logger: logging.Logger = None) -> str:
     proc = subprocess.Popen(
         cmd,
         cwd=cwd,
+        stdin=subprocess.DEVNULL,
         stdout=subprocess.PIPE,
         stderr=subprocess.PIPE,
         env=os.environ,
     )
     stdout, stderr = proc.communicate()
     if proc.returncode:
+        msg = 'ERROR\n'
         if stderr:
-            msg = 'ERROR\n {} '.format(stderr.decode('utf-8').strip())
+            msg = '{}{} '.format(msg, stderr.decode('utf-8').strip())
         raise RuntimeError(msg)
     if stdout:
         return stdout.decode('utf-8').strip()
@@ -872,40 +910,189 @@ def windows_short_path(path: str) -> str:
     return short_path
 
 
-def create_named_text_file(dir: str, prefix: str, suffix: str) -> str:
+def create_named_text_file(
+    dir: str, prefix: str, suffix: str, name_only: bool = False
+) -> str:
     """
-    Create a named unique file.
+    Create a named unique file, return filename.
+    Flag 'name_only' will create then delete the tmp file;
+    this lets us create filename args for commands which
+    disallow overwriting existing files (e.g., 'stansummary').
     """
     fd = tempfile.NamedTemporaryFile(
-        mode='w+', prefix=prefix, suffix=suffix, dir=dir, delete=False
+        mode='w+', prefix=prefix, suffix=suffix, dir=dir, delete=name_only
     )
     path = fd.name
     fd.close()
     return path
 
 
-def install_cmdstan(version: str = None, dir: str = None) -> bool:
+def install_cmdstan(
+    version: str = None,
+    dir: str = None,
+    overwrite: bool = False,
+    verbose: bool = False,
+) -> bool:
     """
-    Run 'install_cmdstan' -script
+    Download and install a CmdStan release from GitHub by running
+    script ``install_cmdstan`` as a subprocess.  Downloads the release
+    tar.gz file to temporary storage.  Retries GitHub requests in order
+    to allow for transient network outages. Builds CmdStan executables
+    and tests the compiler by building example model ``bernoulli.stan``.
+
+    :param version: CmdStan version string, e.g. "2.24.1".
+        Defaults to latest CmdStan release.
+
+    :param dir: Path to install directory.  Defaults to hidden directory
+        ``$HOME/.cmdstan`` or ``$HOME/.cmdstanpy``, if the latter exists.
+        If no directory is specified and neither of the above directories
+        exist, directory ``$HOME/.cmdstan`` will be created and populated.
+
+    :param overwrite:  Boolean value; when ``True``, will overwrite and
+        rebuild an existing CmdStan installation.  Default is ``False``.
+
+    :param verbose:  Boolean value; when ``True``, output from CmdStan build
+        processes will be streamed to the console.  Default is ``False``.
+
+    :return: Boolean value; ``True`` for success.
     """
     logger = get_logger()
     python = sys.executable
     here = os.path.dirname(os.path.abspath(__file__))
     path = os.path.join(here, 'install_cmdstan.py')
-    cmd = [python, path]
+    cmd = [python, '-u', path]
     if version is not None:
         cmd.extend(['--version', version])
     if dir is not None:
         cmd.extend(['--dir', dir])
+    if overwrite:
+        cmd.extend(['--overwrite', 'TRUE'])
+    if verbose:
+        cmd.extend(['--verbose', 'TRUE'])
     proc = subprocess.Popen(
-        cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=os.environ
+        cmd,
+        stdin=subprocess.DEVNULL,
+        stdout=subprocess.PIPE,
+        stderr=subprocess.PIPE,
+        env=os.environ,
     )
     while proc.poll() is None:
-        output = proc.stdout.readline().decode('utf-8').strip()
-        if output:
-            logger.info(output)
-    proc.communicate()
+        print(proc.stdout.readline().decode('utf-8').strip())
+
+    _, stderr = proc.communicate()
     if proc.returncode:
-        logger.warning('CmdStan installation failed')
+        logger.warning('CmdStan installation failed.')
+        if stderr:
+            logger.warning(stderr.decode('utf-8').strip())
         return False
     return True
+
+
+class MaybeDictToFilePath:
+    """Context manager for json files."""
+
+    def __init__(
+        self, *objs: Union[str, dict, list], logger: logging.Logger = None
+    ):
+        self._unlink = [False] * len(objs)
+        self._paths = [''] * len(objs)
+        self._logger = logger or get_logger()
+        i = 0
+        for obj in objs:
+            if isinstance(obj, dict):
+                data_file = create_named_text_file(
+                    dir=_TMPDIR, prefix='', suffix='.json'
+                )
+                self._logger.debug('input tempfile: %s', data_file)
+                if any(
+                    not item
+                    for item in obj
+                    if isinstance(item, (Sequence, np.ndarray))
+                ):
+                    rdump(data_file, obj)
+                else:
+                    jsondump(data_file, obj)
+                self._paths[i] = data_file
+                self._unlink[i] = True
+            elif isinstance(obj, str):
+                if not os.path.exists(obj):
+                    raise ValueError("File doesn't exist {}".format(obj))
+                self._paths[i] = obj
+            elif isinstance(obj, list):
+                err_msgs = []
+                missing_obj_items = []
+                for j, obj_item in enumerate(obj):
+                    if not isinstance(obj_item, str):
+                        err_msgs.append(
+                            (
+                                'List element {} must be a filename string,'
+                                ' found {}'
+                            ).format(j, obj_item)
+                        )
+                    elif not os.path.exists(obj_item):
+                        missing_obj_items.append(
+                            "File doesn't exist: {}".format(obj_item)
+                        )
+                if err_msgs:
+                    raise ValueError('\n'.join(err_msgs))
+                if missing_obj_items:
+                    raise ValueError('\n'.join(missing_obj_items))
+                self._paths[i] = obj
+            elif obj is None:
+                self._paths[i] = None
+            elif i == 1 and isinstance(obj, (Integral, Real)):
+                self._paths[i] = obj
+            else:
+                raise ValueError('data must be string or dict')
+            i += 1
+
+    def __enter__(self):
+        return self._paths
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        for can_unlink, path in zip(self._unlink, self._paths):
+            if can_unlink and path:
+                try:
+                    os.remove(path)
+                except PermissionError:
+                    pass
+
+
+class TemporaryCopiedFile:
+    """Context manager for tmpfiles, handles spaces in filepath."""
+
+    def __init__(self, file_path: str):
+        self._path = None
+        self._tmpdir = None
+        if ' ' in os.path.abspath(file_path) and platform.system() == 'Windows':
+            base_path, file_name = os.path.split(os.path.abspath(file_path))
+            os.makedirs(base_path, exist_ok=True)
+            try:
+                short_base_path = windows_short_path(base_path)
+                if os.path.exists(short_base_path):
+                    file_path = os.path.join(short_base_path, file_name)
+            except RuntimeError:
+                pass
+
+        if ' ' in os.path.abspath(file_path):
+            tmpdir = tempfile.mkdtemp()
+            if ' ' in tmpdir:
+                raise RuntimeError(
+                    'Unable to generate temporary path without spaces! \n'
+                    + 'Please move your stan file to location without spaces.'
+                )
+
+            _, path = tempfile.mkstemp(suffix='.stan', dir=tmpdir)
+
+            shutil.copy(file_path, path)
+            self._path = path
+            self._tmpdir = tmpdir
+        else:
+            self._path = file_path
+
+    def __enter__(self):
+        return self._path, self._tmpdir is not None
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        if self._tmpdir:
+            shutil.rmtree(self._tmpdir, ignore_errors=True)
