@@ -10,6 +10,8 @@ import unittest
 from multiprocessing import cpu_count
 from time import time
 
+import numpy as np
+
 import pytest
 from testfixtures import LogCapture
 
@@ -160,6 +162,17 @@ class SampleTest(unittest.TestCase):
         )
         self.assertEqual(bern_fit.draws().shape, (100, 2, len(BERNOULLI_COLS)))
 
+        np_scalr_10 = np.int32(10)
+        data_dict = {'N': np_scalr_10, 'y': [0, 1, 0, 0, 0, 0, 0, 0, 0, 1]}
+        bern_fit = bern_model.sample(
+            data=data_dict,
+            chains=2,
+            parallel_chains=2,
+            seed=12345,
+            iter_sampling=100,
+        )
+        self.assertEqual(bern_fit.draws().shape, (100, 2, len(BERNOULLI_COLS)))
+
     def test_init_types(self):
         stan = os.path.join(DATAFILES_PATH, 'bernoulli.stan')
         bern_model = CmdStanModel(stan_file=stan)
@@ -1018,7 +1031,7 @@ class CmdStanMCMCTest(unittest.TestCase):
                 self.assertEqual(diags[key].shape, (100, 2))
                 self.assertEqual(
                     bern_fit.sample.shape, (100, 2, len(BERNOULLI_COLS))
-                    )
+                )
         log.check_present(
             (
                 'cmdstanpy',
@@ -1112,6 +1125,29 @@ class CmdStanMCMCTest(unittest.TestCase):
         self.assertTrue('frac_60' in vars)
         self.assertEqual(vars['frac_60'].shape, (20,))
 
+    def test_variables_issue_361(self):
+        # tests that array ordering is preserved
+        stan = os.path.join(DATAFILES_PATH, 'container_vars.stan')
+        container_vars_model = CmdStanModel(stan_file=stan)
+        chain_1_fit = container_vars_model.sample(
+            chains=1, iter_sampling=4, fixed_param=True
+        )
+        v_2d_arr = chain_1_fit.stan_variable('v_2d_arr')
+        self.assertEqual(v_2d_arr.shape, (4, 2, 3))
+        # stan 1-based indexing vs. python 0-based indexing
+        for i in range(2):
+            for j in range(3):
+                self.assertEqual(v_2d_arr[0, i, j], ((i + 1) * 10) + j + 1)
+        chain_2_fit = container_vars_model.sample(
+            chains=2, iter_sampling=4, fixed_param=True
+        )
+        v_2d_arr = chain_2_fit.stan_variable('v_2d_arr')
+        self.assertEqual(v_2d_arr.shape, (8, 2, 3))
+        # stan 1-based indexing vs. python 0-based indexing
+        for i in range(2):
+            for j in range(3):
+                self.assertEqual(v_2d_arr[0, i, j], ((i + 1) * 10) + j + 1)
+
     def test_validate(self):
         stan = os.path.join(DATAFILES_PATH, 'bernoulli.stan')
         jdata = os.path.join(DATAFILES_PATH, 'bernoulli.data.json')
@@ -1338,6 +1374,51 @@ class CmdStanMCMCTest(unittest.TestCase):
         self.assertEqual(fit.stan_vars_dims['beta'], tuple([2]))
         self.assertEqual(fit.stan_vars_cols['beta'], tuple([7, 8]))
 
+    def test_save_diagnostics(self):
+        stan = os.path.join(DATAFILES_PATH, 'bernoulli.stan')
+        jdata = os.path.join(DATAFILES_PATH, 'bernoulli.data.json')
+        bern_model = CmdStanModel(stan_file=stan)
+        bern_fit = bern_model.sample(
+            data=jdata,
+            chains=2,
+            parallel_chains=2,
+            seed=12345,
+            iter_sampling=200,
+            save_diagnostics=True
+        )
+        for i in range(bern_fit.runset.chains):
+            diagnostics_file = bern_fit.runset.diagnostic_files[i]
+            self.assertTrue(os.path.exists(diagnostics_file))
+
+    def test_save_profile(self):
+        stan = os.path.join(DATAFILES_PATH, 'profile_likelihood.stan')
+        profile_model = CmdStanModel(stan_file=stan)
+        profile_fit = profile_model.sample(
+            chains=2,
+            parallel_chains=2,
+            seed=12345,
+            iter_sampling=200,
+            save_profile=True
+        )
+        for i in range(profile_fit.runset.chains):
+            profile_file = profile_fit.runset.profile_files[i]
+            self.assertTrue(os.path.exists(profile_file))
+
+        profile_fit = profile_model.sample(
+            chains=2,
+            parallel_chains=2,
+            seed=12345,
+            iter_sampling=200,
+            save_diagnostics=True,
+            save_profile=True,
+        )
+
+        for i in range(profile_fit.runset.chains):
+            profile_file = profile_fit.runset.profile_files[i]
+            self.assertTrue(os.path.exists(profile_file))
+            diagnostics_file = profile_fit.runset.diagnostic_files[i]
+            self.assertTrue(os.path.exists(diagnostics_file))
+
 
 if __name__ == '__main__':
     unittest.main()
